import abc
import threading
from _typeshed import Incomplete
from abc import ABC, abstractmethod
from tlc.core.objects.tables.system_tables.indexing import _BlacklistExceptionHandler, _ScanUrl, _UrlIndex
from tlc.core.url import Scheme as Scheme, Url as Url
from typing import Iterator
from watchdog.events import FileSystemEvent as FileSystemEvent, FileSystemEventHandler

logger: Incomplete

class _UrlIndexingWorker(ABC, metaclass=abc.ABCMeta):
    def __init__(self, data_event: threading.Event, blacklist_config: list[_BlacklistExceptionHandler] | None, tag: str = '') -> None: ...
    @property
    def data_event(self) -> threading.Event: ...
    def touch(self) -> None:
        """Update the timestamp of the last activity to prevent the indexer from going idle."""
    @abstractmethod
    def set_index(self, new_index: dict[Url, _UrlIndex]) -> None:
        """Set the new index and notify any waiting threads.

        This method will also reset the idle timeout."""
    @abstractmethod
    def get_index(self) -> dict[Url, _UrlIndex]:
        """Retrieve the last index generated by the worker."""
    @abstractmethod
    def request_complete_reindex(self) -> None: ...
    @abstractmethod
    def start(self) -> None: ...
    @abstractmethod
    def stop(self) -> None: ...
    @property
    @abstractmethod
    def is_running(self) -> bool: ...
    def wait_for_complete_reindex(self, timeout: float | None = None) -> bool:
        """Wait until an indexing cycle has completed.

        :return: True if the cycle completed within the timeout, False otherwise.
        """
    def join(self) -> None: ...
    @abstractmethod
    def add_scan_url(self, url_config: _ScanUrl) -> None:
        """Adds a new URL to the list of URLs to be scanned by the indexer.

        The new URL is guaranteed to be scanned in the next cycle.
        The new URL is not visible in the get_scan_urls() method until the next cycle.
        """
    @abstractmethod
    def remove_scan_url(self, url_config: _ScanUrl) -> None:
        """Removes a URL from the list of URLs to be scanned by the indexer.

        The new URL is not guaranteed to be removed until the next cycle.
        """
    @abstractmethod
    def get_scan_urls(self) -> list[_ScanUrl]: ...

class _CrawlUrlIndexer(threading.Thread, _UrlIndexingWorker):
    """An indexer that crawls a directory and indexes the files in it.

    This is an indexer that repeatedly performs a scan of its directories and indexes the files in it. It runs in
    a separate thread to allow for asynchronous scanning, but has an idle overhead. The indexer can be stopped and
    started again.

    The actual scanning of the directory is done by ScanIterators that wrap around URL-adapters.
    """
    def __init__(self, interval: float, data_event: threading.Event, blacklist_config: list[_BlacklistExceptionHandler], tag: str = '') -> None: ...
    def add_scan_url(self, url_config: _ScanUrl) -> None: ...
    def remove_scan_url(self, url_config: _ScanUrl) -> None: ...
    def handle_pending_scan_urls(self) -> None: ...
    def get_scan_urls(self) -> list[_ScanUrl]: ...
    def run(self) -> None:
        """Method representing the thread's activity.

        Do not call this method directly. Use the start() method instead, which will in turn call this method.
        """
    def handle_scan(self, new_scan: Iterator[_UrlIndex]) -> dict[Url, _UrlIndex] | None:
        """Checks a newly scanned index for changes and returns an optionally updated index"""
    def request_complete_reindex(self) -> None: ...
    def wait_for_complete_reindex(self, timeout: float | None = None) -> bool: ...
    def stop(self) -> None:
        """Method to signal the thread to stop its activity.

        This doesn't terminate the thread immediately, but flags it to exit when it finishes its current iteration.
        """
    @property
    def is_running(self) -> bool: ...
    def get_index(self) -> dict[Url, _UrlIndex]: ...
    def set_index(self, new_index: dict[Url, _UrlIndex]) -> None:
        """Set the new index and notify any waiting threads.

        The new data may have removed entries or even be empty.
        """

class _FileEventHandler(FileSystemEventHandler):
    indexer: Incomplete
    base_dir: Incomplete
    def __init__(self, indexer: _FileSystemObserverUrlIndexer, base_dir: Url) -> None: ...
    def on_modified(self, event: FileSystemEvent) -> None: ...
    def on_created(self, event: FileSystemEvent) -> None: ...
    def on_deleted(self, event: FileSystemEvent) -> None: ...
    def process(self, event: FileSystemEvent) -> None: ...

class _FileSystemObserverUrlIndexer(_UrlIndexingWorker):
    """An indexer that watches directories and re-indexes the files whenever they change.

    This indexer is lighter than crawling since no work is performed unless files are changed.
    """
    def __init__(self, data_event: threading.Event, blacklist_config: list[_BlacklistExceptionHandler], tag: str = '') -> None: ...
    def add_scan_url(self, url_config: _ScanUrl) -> None: ...
    def remove_scan_url(self, url_config: _ScanUrl) -> None: ...
    def get_scan_urls(self) -> list[_ScanUrl]: ...
    def handle_scan(self, new_scan: Iterator[_UrlIndex]) -> dict[Url, _UrlIndex] | None:
        """Checks a newly scanned index for changes and returns an optionally updated index"""
    def request_complete_reindex(self) -> None: ...
    def wait_for_complete_reindex(self, timeout: float | None = None) -> bool: ...
    def stop(self) -> None:
        """Method to signal the Indexer to stop its activity."""
    def start(self) -> None:
        """Method to start the Indexer."""
    @property
    def is_running(self) -> bool: ...
    def get_index(self) -> dict[Url, _UrlIndex]: ...
    def set_index(self, new_index: dict[Url, _UrlIndex]) -> None:
        """Set the new index and notify any waiting threads.

        The new data may have removed entries or even be empty.
        """

class _StaticUrlIndexer(_UrlIndexingWorker):
    """An indexer that scans a fixed set of URLs that are considered static.

    This indexer will need a process restart to update.
    """
    def __init__(self, data_event: threading.Event, blacklist_config: list[_BlacklistExceptionHandler], tag: str = '') -> None: ...
    def get_index(self) -> dict[Url, _UrlIndex]: ...
    def add_scan_url(self, url_config: _ScanUrl) -> None: ...
    def remove_scan_url(self, url_config: _ScanUrl) -> None: ...
    @property
    def is_running(self) -> bool: ...
    def stop(self) -> None: ...
    def start(self) -> None: ...
    def request_complete_reindex(self) -> None: ...
    def wait_for_complete_reindex(self, timeout: float | None = None) -> bool: ...
    def get_scan_urls(self) -> list[_ScanUrl]: ...
    def set_index(self, new_index: dict[Url, _UrlIndex]) -> None: ...
