import abc
from _typeshed import Incomplete
from abc import abstractmethod
from datetime import datetime
from tlc.core.objects.tables.system_tables.indexing import _BlacklistExceptionHandler, _ScanIterator, _ScanUrl, _UrlIndex
from tlc.core.url import Url as Url
from tlc.core.url_adapter_registry import UrlAdapterRegistry as UrlAdapterRegistry
from typing import Generator

logger: Incomplete

class _ProjectScanIterator(_ScanIterator, metaclass=abc.ABCMeta):
    """Base class for ScanIterators that scan 3LC project folder layouts."""
    def __init__(self, scan_urls: list[_ScanUrl], tag: str, blacklist_config: list[_BlacklistExceptionHandler] | None) -> None: ...
    @staticmethod
    def projects_readme() -> str: ...
    @staticmethod
    def default_project_readme() -> str: ...
    @abstractmethod
    def scan_project_url(self, projects_url: Url, depth: int) -> Generator[tuple[Url, datetime], None, None]: ...
    def scan_projects_url(self, projects_url: Url, depth: int, create_default_dirs: bool) -> Generator[tuple[Url, datetime], None, None]: ...
    is_first_scan: bool
    def scan(self) -> Generator[_UrlIndex, None, None]: ...
    def add_scan_url(self, scan_url: _ScanUrl) -> None: ...
    def bid(self, config: _ScanUrl) -> int: ...

class _ProjectRunScanIterator(_ProjectScanIterator):
    """A scan iterator that yields all Runs from a 3LC directory layout.

    Runs are stored in a fixed folder structure:
        - Pattern: <projects_dir>/runs/<run_name>
        - Glob: <projects_dir>/*/runs/*
    """
    def __init__(self, scan_urls: list[_ScanUrl], tag: str, blacklist_config: list[_BlacklistExceptionHandler] | None) -> None: ...
    def scan_project_url(self, project_url: Url, depth: int) -> Generator[tuple[Url, datetime], None, None]: ...
    def scan_runs_url(self, runs_url: Url, depth: int) -> Generator[tuple[Url, datetime], None, None]: ...
    def scan_run_url(self, run_url: Url, depth: int) -> Generator[tuple[Url, datetime], None, None]: ...
    def bid(self, config: _ScanUrl) -> int: ...

class _ProjectTableScanIterator(_ProjectScanIterator):
    """A scan iterator that yields all Tables from a 3LC directory layout.

    Tables are stored in a fixed folder structure:
        - Pattern: <projects_dir>/<project_name>/datasets/<dataset_name>/tables/<table_name>
        - Glob: <projects_dir>/*/datasets/*/tables/*/


    :param scan_urls: A list of URLs of directories to scan.
    """
    def __init__(self, scan_urls: list[_ScanUrl], tag: str, blacklist_config: list[_BlacklistExceptionHandler] | None) -> None: ...
    def scan_project_url(self, project_url: Url, depth: int) -> Generator[tuple[Url, datetime], None, None]: ...
    def scan_datasets_url(self, datasets_url: Url, depth: int) -> Generator[tuple[Url, datetime], None, None]: ...
    def scan_dataset_url(self, dataset_url: Url, depth: int) -> Generator[tuple[Url, datetime], None, None]: ...
    def scan_tables_url(self, tables_url: Url, depth: int) -> Generator[tuple[Url, datetime], None, None]: ...
    def scan_table_url(self, table_url: Url, depth: int) -> Generator[tuple[Url, datetime], None, None]: ...
    def bid(self, config: _ScanUrl) -> int: ...
